/*
 * Title:        sag_internal/c_functions.hpp
 * Description:  C function ABI
 * $Copyright (c) 2015-2017 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 * @Version:     $Id: c_functions.hpp 320247 2017-11-17 14:42:32Z asd $
 */

#ifndef _SAG_INTERNAL_C_FUNCTIONS_HPP_
#define _SAG_INTERNAL_C_FUNCTIONS_HPP_

// already in extern "C"

/* @private
 * @file Contains the imports/exports for the free functions implemented by the host (for use by plugins), 
 * and typedefs for the function signatures implemented and exported by plugins. These are implemented in CConnectivity
 * 
 *
 * It's important that memory allocation is done all in the same allocator
 * realm, so all copying, allocation and free must be done in the correlator.
 * These C free functions allow this to be done over the C API
 */

/** Deep copy a data_t. */
SAG_CONNECTIVITY_API sag_underlying_data_t SAG_CONNECTIVITY_CALL sag_copy_data(sag_underlying_data_t);

/** Allocate a vector of (at least) the given size */
SAG_CONNECTIVITY_API sag_underlying_vector_t SAG_CONNECTIVITY_CALL sag_allocate_vector(uint64_t);

/** Allocate a new vector of (at least) the given size and shallow-copy the
	old vector into it, then free the old vector. */
SAG_CONNECTIVITY_API sag_underlying_vector_t SAG_CONNECTIVITY_CALL sag_reallocate_vector(sag_underlying_vector_t, uint64_t);

/** Free the provided vector */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_free_vector(sag_underlying_vector_t);

/** Allocate a map of (at least) the given size */
SAG_CONNECTIVITY_API sag_underlying_map_t SAG_CONNECTIVITY_CALL sag_allocate_map(uint64_t);

/** Allocate a new map of (at least) the given size and shallow-copy the old
	map into it, then free the old map */
SAG_CONNECTIVITY_API sag_underlying_map_t SAG_CONNECTIVITY_CALL sag_reallocate_map(sag_underlying_map_t, uint64_t);

/** Free the given map */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_free_map(sag_underlying_map_t);

/** Copy the given string */
SAG_CONNECTIVITY_API const char * SAG_CONNECTIVITY_CALL sag_copy_string(const char *);

/** Copy the given string, but only the first n bytes */
SAG_CONNECTIVITY_API const char * SAG_CONNECTIVITY_CALL sag_copy_partial_string(const char *, int64_t);

/** Returns the hash of the given string */
SAG_CONNECTIVITY_API uint64_t SAG_CONNECTIVITY_CALL sag_hash_string(const char *);

/** Free the given string, parameter must be value returned from sag_copy_string */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_free_string(const char *);

/** Allocate a byte buffer */
SAG_CONNECTIVITY_API sag_underlying_buffer_t SAG_CONNECTIVITY_CALL sag_allocate_buffer(uint64_t);

/** Free a byte buffer */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_free_buffer(sag_underlying_buffer_t);

/** Copy a byte buffer */
SAG_CONNECTIVITY_API sag_underlying_buffer_t SAG_CONNECTIVITY_CALL sag_copy_buffer(sag_underlying_buffer_t);

/** Allocate a custom */
SAG_CONNECTIVITY_API sag_underlying_custom_t SAG_CONNECTIVITY_CALL sag_allocate_custom(void *, void (*)(void*), void *(*)(void*));

/** Free a custom */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_free_custom(sag_underlying_custom_t);

/** Copy a custom */
SAG_CONNECTIVITY_API sag_underlying_custom_t SAG_CONNECTIVITY_CALL sag_copy_custom(sag_underlying_custom_t, bool shallow = false);

/** Log a message to the system log file at the given level */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_log_at(const char*, sag_log_level_t, const char *, ...);

/** Returns the current log level for the given category */
SAG_CONNECTIVITY_API sag_log_level_t SAG_CONNECTIVITY_CALL sag_get_log_level(const char *);


/** Create a chain */
SAG_CONNECTIVITY_API sag_plugin_t SAG_CONNECTIVITY_CALL sag_create_chain(void*, const char *, const char *, const char *, sag_underlying_vector_t, sag_underlying_vector_t, sag_underlying_map_t, void*);
/** Start a created chain */
SAG_CONNECTIVITY_API sag_error_t SAG_CONNECTIVITY_CALL sag_start_chain(void*, sag_plugin_t);
/** Destroy a chain */
SAG_CONNECTIVITY_API sag_error_t SAG_CONNECTIVITY_CALL sag_destroy_chain(void*, sag_plugin_t);

/** Create a user-defined status value */
SAG_CONNECTIVITY_API sag_status_item_t SAG_CONNECTIVITY_CALL sag_create_user_status_item(void* connectivityManager, const char*, const char*);
/** Delete a user-defined status value */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_delete_user_status_item(void* connectivityManager, sag_status_item_t);
/** Set the current value of a user-defined status value */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_set_user_status_item(sag_status_item_t, const char*);

/** Set user-defined status values from a string,string map */
SAG_CONNECTIVITY_API sag_error_t SAG_CONNECTIVITY_CALL sag_set_user_status_map(void*, const sag_underlying_map_t);
/** Enable reliability */
SAG_CONNECTIVITY_API sag_error_t SAG_CONNECTIVITY_CALL sag_enable_reliability(void* chain, int direction);

/** Set the thread cache on this thread */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_set_thread_cache(void *);
/** destroy a thread cache */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL sag_destroy_thread_cache(void *);
/** create a thread cache */
SAG_CONNECTIVITY_API void* SAG_CONNECTIVITY_CALL sag_create_thread_cache();

/** Fn pointer type for send batch functions */
typedef sag_error_t (*sag_send_fn_t)(sag_plugin_t, sag_underlying_message_t *, sag_underlying_message_t *);
/** Fn pointer type for create plugin functions, considered legacy from 9.12.0.1 onwards (we retain this only to preserve binary compat between 9.12.0.0 and 9.12.0.1) */
typedef sag_plugin_t (*sag_create_fn_t)(const char *, const char *, sag_underlying_data_t);
/** Fn pointer type for create plugin functions, available from 9.12.0.1 onwards */
typedef sag_plugin_t(*sag_create_with_params_fn_t)(const char *, const char *, sag_underlying_data_t, void*, void*);
/** Fn pointer type for sag_chainmanager_create functions */
typedef sag_plugin_t (*sag_chainmanager_create_fn_t)(sag_underlying_data_t config, const char * managerName, const char * transportPluginName, void* connectivityManager, sag_underlying_data_t chainDefs, void* reserved);
/** Fn pointer type for chainmanager_createTransport function */
typedef sag_plugin_t (*sag_chainmanager_createTransport_fn_t)(sag_plugin_t managerplugin, const char *chainId,
                       sag_underlying_data_t config, void* connectivityManager, const char* defaultChannelTowardsHost, sag_underlying_vector_t subscribeChannels, void *createFun, void* reserved);
/** Fn pointer type for start, shutdown, hostReady and destroy functions */
typedef sag_error_t (*sag_status_fn_t)(sag_plugin_t);
/** Fn pointer type for setNext functions */
typedef sag_error_t (*sag_set_next_fn_t)(sag_plugin_t, sag_plugin_t, sag_send_fn_t);
/** Fn pointer type for api_version functions */
typedef uint64_t (*sag_version_fn_t)();
/** Fn pointer type for returning the (static string constant) for the 4-digit product version number */
typedef char* (*sag_product_version_fn_t)();
/** Fn pointer type for channel listener callbacks */
typedef sag_error_t (*sag_channel_fn_t)(const char *, sag_plugin_t listener, const char *, bool);

/** Add a listener for channels */
SAG_CONNECTIVITY_API sag_error_t SAG_CONNECTIVITY_CALL sag_add_channel_listener(const char *, void*, sag_plugin_t, const char *, sag_status_fn_t, sag_channel_fn_t, sag_channel_fn_t);

/** Check if host is shutting down. */
SAG_CONNECTIVITY_API sag_error_t SAG_CONNECTIVITY_CALL sag_is_host_shutting_down(void* chain, bool &isShuttingDown);

SAG_CONNECTIVITY_API sag_underlying_data_t SAG_CONNECTIVITY_CALL sag_data_to_string(sag_underlying_data_t data);

#endif // _SAG_INTERNAL_C_FUNCTIONS_HPP_
